"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildConfigFromConnections = buildConfigFromConnections;
exports.buildConnectionsDictFromList = buildConnectionsDictFromList;
exports.buildConfigString = buildConfigString;
/**
 * 1. Receive connections list, and transform it into the connections dictionary.
 *   - Store only connectionId, name, and slug.
 * 2. Create the config object from a default config.
 * 3. Create a connections environments array by making a filtered array of connections for each environment.
 * 4. Map each object in the environments array to expand out
 */
const path_1 = __importDefault(require("path"));
function buildConfigFromConnections(connections, shouldCreateMappingTemplates) {
    const environments = getUniqueConnectionEnvs(connections);
    return {
        helpers: path_1.default.join("src", "templates", "helpers.jsonata"),
        environments: buildTemplateEnvironments(environments, connections, shouldCreateMappingTemplates),
    };
}
function buildConnectionsDictFromList(connections) {
    return connections.reduce((dict, conn) => {
        if (conn.name) {
            dict[conn.name] = {
                id: conn.id,
                slug: conn.slug,
                environment: conn.environment,
            };
        }
        return dict;
    }, {});
}
function getUniqueConnectionEnvs(connections) {
    return connections.reduce((envs, conn) => {
        if (!envs.includes(conn.environment))
            envs.push(conn.environment);
        return envs;
    }, []);
}
function buildTemplateEnvironments(environments, connections, shouldCreateMappingTemplates) {
    return environments.map((env) => {
        return {
            environment: env,
            templates: [
                {
                    connection_ids: connections
                        .filter((conn) => conn.environment === env)
                        .map((conn) => conn.id),
                    paths: {
                        item: path_1.default.join("src", "templates", "item", "item.jsonata"),
                        variation: path_1.default.join("src", "templates", "variation", "variation.jsonata"),
                        item_group: path_1.default.join("src", "templates", "item_group", "item_group.jsonata"),
                        ...(shouldCreateMappingTemplates && {
                            mapping: path_1.default.join("src", "templates", "mapping", "mapping.jsonata"),
                        }),
                    },
                },
            ],
        };
    });
}
function buildConfigString(config, connections) {
    let configString = JSON.stringify(config, null, 2);
    for (const connectionName in connections) {
        const conn = connections[connectionName];
        /**
         * We need to escape the name to ensure the JSON won't break, in case
         * there are special characters in the connection name.
         */
        const escapedName = JSON.stringify(connectionName);
        configString = configString.replace(`"${conn.id}"`, `connections[${escapedName}].id`);
    }
    return configString;
}
